# lib-bb-portal-rest


Version: **1.0.0**

Provides a cross-platform SDK for Backbase REST APIs

## Example

```javascript
import bbrest from 'lib-bb-portal-rest';
import ajaxAdapter from 'lib-bb-portal-rest-ajax-adapter';

const BBRest = bbrest(ajaxAdapter, { port: 8080 });

BBRest.importPackage().file('item.zip').post()
  .then(xmlString => {
    // do something
  });
```

## Table of Contents
- **Exports**<br/>    <a href="#default">default(adapter, restConfig, middleware)</a><br/>
- **Type Definitions**<br/>    <a href="#Adapter">Adapter</a><br/>    <a href="#RestConfig">RestConfig</a><br/>    <a href="#RequestObject">RequestObject</a><br/>    <a href="#Middleware">Middleware(adapter, restConfig)</a><br/>    <a href="#ConfiguredMiddleware">ConfiguredMiddleware(reqObj)</a><br/>    <a href="#CXPError">CXPError</a><br/>    <a href="#Response">Response</a><br/>

## Exports


### <a name="default"></a>*default(adapter, restConfig, middleware)*

BBRest constructor

| Parameter | Type | Description |
| :-- | :-- | :-- |
| adapter | <a href="#Adapter">Adapter</a> | js module exposing cross platform methods |
| restConfig | <a href="#RestConfig">RestConfig</a> | Configuration object |
| middleware | Array of <a href="#Middleware">Middleware</a> (optional) | Array of middleware functions. |

##### Returns

<a href="#BBRest">BBRest</a> - *BBRest API*

---

## BBRest

BBRest API
Each of the following chained methods will return Promise
that resolves either to <a href="#Response">Response</a> or rejects with <a href="#CXPError">CXPError</a>

## Example

```javascript
.server().get();
.server().post(portal);
.server().put(portal);
.catalog().get();
.catalog().post(catalog);
.catalog().put(catalog);
.catalog().delete(catalog);
.catalog('my-item').get();
.catalog('my-item').delete();
.catalog().tags().get();
.template().get();
.template().post(template);
.template('my-template').get();
.template('my-template').put(template);
.template('my-template').rights().get();
.template('my-template').rights().put(rights);
.feature().get();
.feature().post(feature);
.feature('my-feature').get();
.feature('my-feature').put(feature);
.feature('my-feature').rights().get();
.feature('my-feature').rights().put(rights);
.user().get();
.user().post(user);
.user('my-user').get();
.user('my-user').put(user);
.user('my-user').delete();
.user('my-user', true).get();
.user('my-user', true).post(groups);
.user('my-user', true, 'my-group').delete();
.group().get();
.group().post(group);
.group('my-group').get();
.group('my-group').put(group);
.group('my-group').delete();
.group('my-group', true).get();
.group('my-group', true).post(users);
.group('my-group', true, 'my-user').delete();
.audit().get();
.audit(true).get();
.cache('my-cache').delete();
.import().post(exportedPortal);
.import().file('/my-portal.zip').post();
.importPackage().file('/my-package.zip').post();
.export().post(portalExportRequest);
.export('uuid').file('/my-portal.zip').get();
.exportPackage('my-package').file('/my-package.zip').get();
.provision().file('/my-package.zip').post();
.provision('uuid').get();
.provision('uuid', true).file('/my-package.zip').get();
.portal('my-portal').get();
.portal('my-portal').put(portal);
.portal('my-portal').delete();
.portal('my-portal').rights().get();
.portal('my-portal').rights().put(rights);
.portal('my-portal').tags().get();
.portal('my-portal').tags().post(tag);
.portal('my-portal').tags('my-tag').delete();
.portal('my-portal').catalog().get();
.portal('my-portal').catalog().post(catalog);
.portal('my-portal').catalog().put(catalog);
.portal('my-portal').catalog().delete(catalog);
.portal('my-portal').catalog('my-item').get();
.portal('my-portal').catalog('my-item').delete();
.portal('my-portal').catalog().tags().get();
.portal('my-portal').page().get();
.portal('my-portal').page().post(page);
.portal('my-portal').page().put(pages);
.portal('my-portal').page('my-page').get();
.portal('my-portal').page('my-page').put(page);
.portal('my-portal').page('my-page').delete();
.portal('my-portal').page('my-page').rights().get();
.portal('my-portal').page('my-page').rights().put(rights);
.portal('my-portal').container().get();
.portal('my-portal').container().post(container);
.portal('my-portal').container().put(containers);
.portal('my-portal').container('my-container').get();
.portal('my-portal').container('my-container').put(container);
.portal('my-portal').container('my-container').delete();
.portal('my-portal').container('my-container').rights().get();
.portal('my-portal').container('my-container').rights().put(rights);
.portal('my-portal').widget().get();
.portal('my-portal').widget().post(widget);
.portal('my-portal').widget().put(widgets);
.portal('my-portal').widget('my-widget').get();
.portal('my-portal').widget('my-widget').put(widget);
.portal('my-portal').widget('my-widget').delete();
.portal('my-portal').widget('my-widget').rights().get();
.portal('my-portal').widget('my-widget').rights().put(rights);
.portal('my-portal').link().get();
.portal('my-portal').link().post(link);
.portal('my-portal').link().put(links);
.portal('my-portal').link().delete(links);
.portal('my-portal').link('my-link').get();
.portal('my-portal').link('my-link').put(link);
.portal('my-portal').link('my-link').delete();
.portal('my-portal').link('my-link').rights().get();
.portal('my-portal').link('my-link').rights().put(rights);
.portal('my-portal').importPackage().file('/my-package.zip').post();
.portal('my-portal').export().get();
.portal('my-portal').export().post();
.portal('my-portal').exportPackage('my-package').file('/my-package.zip').get();

bbrest.portal('my-portal').export().post()
  .then(body => {
    const id = formalize(body).model.identifier;
    return bbrest.export(id).file('my-portal.zip').get();
  });
```

## Type Definitions


### <a name="Adapter"></a>*Adapter*

Platform specific adapter module

**Type:** *Object*


| Property | Type | Description |
| :-- | :-- | :-- |
| request | Function | http request method |
| log | Function | log method |
| btoa | Function | base64 string convert method |
| DOMParser | Function | DOM parser implementation |

### <a name="RestConfig"></a>*RestConfig*

Rest API Configuration

**Type:** *Object*


| Property | Type | Description |
| :-- | :-- | :-- |
| scheme | String | url scheme |
| host | String | url host |
| port | String | url port |
| context | String | url path prefix |
| username | String | username |
| password | String | password |
| authPath | String | url path for authorization |
| orchestrator | String | path of the orchestrator |
| version | String | portal version |

### <a name="RequestObject"></a>*RequestObject*


**Type:** *Object*


| Property | Type | Description |
| :-- | :-- | :-- |
| url | String | relative url path of http request |
| method | String | http method |
| query | Object | query hash |
| headers | Object | headers hash |
| importFile | String | NodeJS: path of the file to upload, browser: Blob object to upload |
| exportFile | String | file path (NodeJS) or name (browser) to download to |
| formField | String | name of the form field for upload |


### <a name="Middleware"></a>*Middleware(adapter, restConfig)*

Middleware functions are used to change http request before it is executed.

| Parameter | Type | Description |
| :-- | :-- | :-- |
| adapter | <a href="#Adapter">Adapter</a> | js module exposing cross platform methods |
| restConfig | <a href="#RestConfig">RestConfig</a> | Configuration object |

##### Returns

<a href="#ConfiguredMiddleware">ConfiguredMiddleware</a> - *function that runs middleware*


### <a name="ConfiguredMiddleware"></a>*ConfiguredMiddleware(reqObj)*

Function that executes middleware.

| Parameter | Type | Description |
| :-- | :-- | :-- |
| reqObj | <a href="#RequestObject">RequestObject</a> | Request object |

##### Returns

<a href="#RequestObject">RequestObject</a> - *Request object to execute*

### <a name="CXPError"></a>*CXPError*


**Type:** *Error*


| Property | Type | Description |
| :-- | :-- | :-- |
| message | String | error message |
| response | <a href="#Response">Response</a> | http response object |
| request | <a href="#RequestObject">RequestObject</a> | http request object |
| restConfig | <a href="#RestConfig">RestConfig</a> | REST API configuration |

### <a name="Response"></a>*Response*


**Type:** *Object*


| Property | Type | Description |
| :-- | :-- | :-- |
| status | Number | http status code |
| statusText | String | http status text |
| headers | Object | http response headers |
| body | String | http response body |

---
